@@ -131,9 +131,11 @@ CheckIssuer(Input encodedIssuer)
 {
   // "The issuer field MUST contain a non-empty distinguished name (DN)."
   Reader issuer(encodedIssuer);
-  Input encodedRDNs;
-  ExpectTagAndGetValue(issuer, der::SEQUENCE, encodedRDNs);
-  Reader rdns(encodedRDNs);
+  Reader rdns;
+  Result rv = der::ExpectTagAndGetValueAtEnd(issuer, der::SEQUENCE, rdns);
+  if (rv != Success) {
+    return rv;
+  }
   // Check that the issuer name contains at least one RDN
   // (Note: this does not check related grammar rules, such as there being one
   // or more AVAs in each RDN, or the values in AVAs not being empty strings)
@@ -420,7 +422,7 @@ CheckKeyUsage(EndEntityOrCA endEntityOrCA, const Input* encodedKeyUsage,
 
   Reader input(*encodedKeyUsage);
   Reader value;
-  if (der::ExpectTagAndGetValue(input, der::BIT_STRING, value) != Success) {
+  if (der::ExpectTagAndGetValueAtEnd(input, der::BIT_STRING, value) != Success) {
     return Result::ERROR_INADEQUATE_KEY_USAGE;
   }
 
@@ -725,9 +727,7 @@ CheckBasicConstraints(EndEntityOrCA endEntityOrCA,
 
 static Result
 MatchEKU(Reader& value, KeyPurposeId requiredEKU,
-         EndEntityOrCA endEntityOrCA, TrustDomain& trustDomain,
-         Time notBefore, /*in/out*/ bool& found,
-         /*in/out*/ bool& foundOCSPSigning)
+         /*in/out*/ bool& found, /*in/out*/ bool& foundOCSPSigning)
 {
   // See Section 5.9 of "A Layman's Guide to a Subset of ASN.1, BER, and DER"
   // for a description of ASN.1 DER encoding of OIDs.
@@ -747,34 +747,13 @@ MatchEKU(Reader& value, KeyPurposeId requiredEKU,
   static const uint8_t email [] = { (40*1)+3, 6, 1, 5, 5, 7, 3, 4 };
   static const uint8_t ocsp  [] = { (40*1)+3, 6, 1, 5, 5, 7, 3, 9 };
 
-  // id-Netscape        OBJECT IDENTIFIER ::= { 2 16 840 1 113730 }
-  // id-Netscape-policy OBJECT IDENTIFIER ::= { id-Netscape 4 }
-  // id-Netscape-stepUp OBJECT IDENTIFIER ::= { id-Netscape-policy 1 }
-  static const uint8_t serverStepUp[] =
-    { (40*2)+16, 128+6,72, 1, 128+6,128+120,66, 4, 1 };
-
   bool match = false;
 
   if (!found) {
     switch (requiredEKU) {
-      case KeyPurposeId::id_kp_serverAuth: {
-        if (value.MatchRest(server)) {
-          match = true;
-          break;
-        }
-        // Potentially treat CA certs with step-up OID as also having SSL server
-        // type. Comodo has issued certificates that require this behavior that
-        // don't expire until June 2020!
-        if (endEntityOrCA == EndEntityOrCA::MustBeCA &&
-            value.MatchRest(serverStepUp)) {
-          Result rv = trustDomain.NetscapeStepUpMatchesServerAuth(notBefore,
-                                                                  match);
-          if (rv != Success) {
-            return rv;
-          }
-        }
+      case KeyPurposeId::id_kp_serverAuth:
+        match = value.MatchRest(server);
         break;
-      }
 
       case KeyPurposeId::id_kp_clientAuth:
         match = value.MatchRest(client);
@@ -815,8 +794,7 @@ MatchEKU(Reader& value, KeyPurposeId requiredEKU,
 Result
 CheckExtendedKeyUsage(EndEntityOrCA endEntityOrCA,
                       const Input* encodedExtendedKeyUsage,
-                      KeyPurposeId requiredEKU, TrustDomain& trustDomain,
-                      Time notBefore)
+                      KeyPurposeId requiredEKU)
 {
   // XXX: We're using Result::ERROR_INADEQUATE_CERT_TYPE here so that callers
   // can distinguish EKU mismatch from KU mismatch from basic constraints
@@ -831,8 +809,7 @@ CheckExtendedKeyUsage(EndEntityOrCA endEntityOrCA,
     Reader input(*encodedExtendedKeyUsage);
     Result rv = der::NestedOf(input, der::SEQUENCE, der::OIDTag,
                               der::EmptyAllowed::No, [&](Reader& r) {
-      return MatchEKU(r, requiredEKU, endEntityOrCA, trustDomain, notBefore,
-                      found, foundOCSPSigning);
+      return MatchEKU(r, requiredEKU, found, foundOCSPSigning);
     });
     if (rv != Success) {
       return Result::ERROR_INADEQUATE_CERT_TYPE;
@@ -914,7 +891,7 @@ TLSFeaturesSatisfiedInternal(const Input* requiredTLSFeatures,
   const static uint8_t status_request_bytes[] = { status_request };
 
   Reader input(*requiredTLSFeatures);
-  return der::NestedOf(input, der::SEQUENCE, der::INTEGER,
+  Result rv = der::NestedOf(input, der::SEQUENCE, der::INTEGER,
                        der::EmptyAllowed::No, [&](Reader& r) {
     if (!r.MatchRest(status_request_bytes)) {
       return Result::ERROR_REQUIRED_TLS_FEATURE_MISSING;
@@ -926,6 +903,10 @@ TLSFeaturesSatisfiedInternal(const Input* requiredTLSFeatures,
 
     return Result::Success;
   });
+  if (rv != Success) {
+    return rv;
+  }
+  return der::End(input);
 }
 
 Result
@@ -1065,7 +1046,7 @@ CheckIssuerIndependentProperties(TrustDomain& trustDomain,
 
   // 4.2.1.12. Extended Key Usage
   rv = CheckExtendedKeyUsage(endEntityOrCA, cert.GetExtKeyUsage(),
-                             requiredEKUIfPresent, trustDomain, notBefore);
+                             requiredEKUIfPresent);
   if (rv != Success) {
     return rv;
   }
